require './test/test_helper'

require 'hover/client/hmac'

describe Hover::Client::HMAC do
  let(:client) { Hover::Client::HMAC.new(1, 2, 'http://localhost') }
  let(:path) { '/testing' }
  let(:request) { mock }
  let(:response) { mock }

  describe "authenticate" do
    it "authenticate" do
      request = mock
      ApiAuth.expects(:sign!).with(request, 1, 2).once

      client.authenticate(request)
    end
  end

  describe "make_uri" do
    it "makes correct uri with parameters" do
      client.make_uri('testing', {a: 1}).to_s.must_equal("http://localhost/api/v1/testing?a=1")
    end

    it "with parameters" do
      URI.expects(:parse).twice
      URI.expects(:encode_www_form).with({a: 1}).once

      client.make_uri('testing', {a: 1})
    end

    it "makes correct uri without parameters" do
      client.make_uri('testing').to_s.must_equal("http://localhost/api/v1/testing")
    end

    it "without parameters" do
      URI.expects(:parse).once
      Array.any_instance.expects(:join).once.returns('http://localhost/api/v1/testing')
      URI.expects(:encode_www_form).never

      client.make_uri('testing')
    end
  end

  describe "make_request" do
    it "requests without ssl" do
      uri = URI('http://localhost/test')
      http = mock

      http.expects(:request).with(request).returns(response).once
      http.expects(:use_ssl=).never

      client.expects(:raise_errors_for_cloudfront_response)
      client.expects(:authenticate).with(request).once
      Net::HTTP.expects(:new).with(uri.host, uri.port).returns(http)

      client.make_request(request, uri).must_equal(response)
    end

    it 'requests with ssl' do
      uri = URI('https://localhost/test')
      http = mock

      http.expects(:request).with(request).returns(response).once
      http.expects(:use_ssl=).with(true).once

      client.expects(:raise_errors_for_cloudfront_response)
      client.expects(:authenticate).with(request).once
      Net::HTTP.expects(:new).with(uri.host, uri.port).returns(http)

      client.make_request(request, uri).must_equal(response)
    end
  end

  describe "get" do
    it "gets without parameters" do
      uri = URI("http://localhost/testing")
      Net::HTTP::Get.expects(:new).with(uri).returns(request)
      client.expects(:make_request).with(request, uri).returns(response)

      client.expects(:make_uri).with(path, {}).once.returns(uri)

      client.get(path)
    end

    it "gets with parameters" do
      parameters = {a: 1, b: 2}
      uri = URI("http://localhost/testing?a=1&b=2")
      Net::HTTP::Get.expects(:new).with(uri).returns(request)
      client.expects(:make_request).with(request, uri).returns(response)

      client.expects(:make_uri).with(path, parameters).once.returns(uri)

      client.get(path, parameters)
    end
  end

  describe "get_redirect_location" do
    let(:uri) { URI("http://localhost/testing") }

    before do
      client.expects(:parse_response).never

      Net::HTTP::Get.expects(:new).with(uri).returns(request)
      client.expects(:make_request).with(request, uri).returns(response)
      client.expects(:make_uri).with(path, {}).once.returns(uri)
    end

    it "isn't a redirect response" do
      response.expects(:is_a?).with(Net::HTTPRedirection).returns(false)
      response.expects(:[]).with('location').never

      assert_nil client.get_redirect_location(path)
    end

    it "is a redirect response, but no location was given" do
      response.expects(:is_a?).with(Net::HTTPRedirection).returns(true)
      response.expects(:[]).with('location').once.returns(nil)

      assert_nil client.get_redirect_location(path)
    end

    it "is a redirect" do
      redirect_location = 'http://location/elsewhere'

      response.expects(:is_a?).with(Net::HTTPRedirection).returns(true)
      response.expects(:[]).with('location').returns(redirect_location).twice

      client.get_redirect_location(path).must_equal(redirect_location)
    end

  end

  describe "post" do
    it 'posts' do
      path = '/post'
      uri = URI.parse('http://localhost/post')

      client.expects(:make_uri).with(path).returns(uri).once
      Net::HTTP::Post.expects(:new).with(uri).returns(request)

      client.expects(:make_request).with(request, uri).returns(response)
      request.expects(:set_form_data).with({}).once

      client.post(path)
    end
  end

  describe "put" do
    it 'puts' do
      path = '/put'
      uri = URI.parse('http://localhost/put')

      client.expects(:make_uri).with(path).returns(uri).once
      Net::HTTP::Put.expects(:new).with(uri).returns(request)

      client.expects(:make_request).with(request, uri).returns(response)
      request.expects(:set_form_data).with({}).once

      client.put(path)
    end
  end

  describe "patch" do
    it 'patch' do
      path = '/patch'
      uri = URI.parse('http://localhost/patch')

      client.expects(:make_uri).with(path).returns(uri).once
      Net::HTTP::Patch.expects(:new).with(uri).returns(request)

      client.expects(:make_request).with(request, uri).returns(response)
      request.expects(:set_form_data).with({}).once

      client.patch(path)
    end
  end

  describe "delete" do
    it 'delete' do
      path = '/delete'
      uri = URI.parse('http://localhost/delete')

      client.expects(:make_uri).with(path).returns(uri).once
      Net::HTTP::Delete.expects(:new).with(uri).returns(request)

      client.expects(:make_request).with(request, uri).returns(response)
      request.expects(:set_form_data).with({}).once

      client.delete(path)
    end
  end

  describe "parse_response" do
    let(:response) { mock }
    let(:valid_json) { '{"a": 1}' }
    let(:invalid_json) { '{"a": 1, "b" => 2}' }

    before do
      client.expects(:raise_errors).with(response).once
    end

    it "204 no content" do
      response.expects(:code).returns(204)
      response.expects(:body).never
      ::JSON.expects(:parse).never

      client.parse_response(response)
    end

    it "valid json" do
      response.expects(:code).returns(200)
      response.expects(:body).returns(valid_json).once
      ::JSON.expects(:parse).with(valid_json)

      client.parse_response(response)
    end

    it "invalid json" do
      response.expects(:code).returns(200)
      response.expects(:body).returns(invalid_json).once
      ::JSON.expects(:parse).with(invalid_json)

      client.parse_response(response)
    end

  end

  describe "raise_errors_for_cloudfront_response" do
    let(:header) { mock }
    let(:cloudfront_server) { 'Cloudfront' }
    let(:nginx_server) { 'nginx/1.2.9' }

    before do
      response.expects(:header).returns(header)
    end

    it "cloudfront" do
      header.expects(:[]).with('Server').returns(cloudfront_server)

      ->{ client.raise_errors_for_cloudfront_response(response) }.must_raise(Hover::Client::Unavailable)
    end

    it "nginx" do
      header.expects(:[]).with('Server').returns(nginx_server)

      client.raise_errors_for_cloudfront_response(response)
    end
  end

  describe "raise_errors" do
    before do
      response.expects(:body).once
    end

    {
      400 => Hover::Client::BadRequest,
      401 => Hover::Client::Unauthorized,
      403 => Hover::Client::General,
      404 => Hover::Client::NotFound,
      500 => Hover::Client::InternalError,
      502 => Hover::Client::Unavailable,
      503 => Hover::Client::Unavailable
    }.each do |code, exception_class|
      it code.to_s do
        response.expects(:code).returns(code.to_s).twice

        -> { client.raise_errors(response) }.must_raise(exception_class)
      end
    end

    it "200" do
      response.expects(:code).returns('200').twice
      client.raise_errors(response)
    end
  end
end