require './test/test_helper'

require 'hover/encoder/json_stream'

describe Hover::Encoder::JSONStream do
  let(:instance) { Hover::Encoder::JSONStream.new }

  let(:attributes) do
    {'a' => 1, 'b' => 2}
  end

  it "initializes" do
    instance.must_be_kind_of(Hover::Encoder::JSONStream)
  end

  describe "open" do
    it "sets file" do
      file = mock
      File.expects(:open).with("/tmp/filename", 'w').returns(file)

      instance.open("/tmp/filename")
      instance.file.must_equal(file)
    end
  end

  describe "close" do
    it "closes file" do
      file = mock
      file.expects(:close)

      instance.file = file
      instance.close

      assert_nil instance.file
    end
  end

  describe "append" do
    it "appends" do
      file = mock
      file.expects(:puts).with("#{attributes.merge({'class' => 'Hash'}).to_json}")

      instance.file = file

      instance.append(attributes, attributes.class)
    end

    it "last" do
      file = mock
      file.expects(:puts).with("#{attributes.merge({'class' => 'Hash'}).to_json}")

      instance.file = file

      instance.append(attributes, attributes.class)
    end
  end

  describe "append_each" do
    it "appends" do
      file = mock
      file.expects(:puts).with("#{attributes.merge({'class' => 'Hash'}).to_json}").twice

      instance.file = file

      instance.append_each([attributes, attributes], attributes.class)
    end
  end

  describe "append_records" do
    it "appends from scope" do
      TestClass.destroy_all

      a = TestClass.create!
      b = TestClass.create!

      instance.expects(:append).with(a.attributes, TestClass)
      instance.expects(:append).with(b.attributes, TestClass)

      instance.append_records(TestClass.all)
    end
  end

  describe "valid json" do

    it "avoids NaN problem" do
      # with PostgreSQL we could use TestClass#a, but sqlite persists the NaN as nil
      class SpecialStandInForThisTest
        def find_each
          yield self
        end
        def attributes
          # if JSONStream doesn't turn this NaN into a nil we'll see an error in parsing the generated JSON
          nan = 0.0 / 0.0
          {"foo" => "bar", "x" => nan}
        end
      end
      file = Tempfile.new(['hover-ruby-client_json_encoder-NaN', '.json'])
      instance.open(file.path)
      instance.append_records(SpecialStandInForThisTest.new)
      instance.close
      JSON.parse(file.read).must_equal({"foo" => "bar", "x" => nil, "class"=>"SpecialStandInForThisTest"})
    end

    it "works" do
      file = Tempfile.new(['hover-ruby-client_json_encoder', '.json'])

      begin
        1.upto(50) do
          TestClass.create!
        end

        1.upto(50) do
          Tag.create!
        end

        instance.open(file.path)
        instance.append_records(TestClass.all)
        instance.append_records(Tag.all)
        instance.close

        expected = [
          TestClass.all.map { |test_class| test_class.attributes.merge({'class' => 'TestClass'}) },
          Tag.all.map { |tag| tag.attributes.merge({'class' => 'Tag'}) },
        ].flatten

        file.each_line do |line|
          expected.include?(JSON.parse(line)).must_equal(true)
        end
      ensure
        file.close
        file.unlink
      end
    end
  end
end